@@ -1,5 +1,3 @@
-# coding: utf-8
-from __future__ import unicode_literals
 from defusedxml.xmlrpc import DefusedExpatParser
 from devpi_common.metadata import Version
 from devpi_common.metadata import get_pyversion_filetype
@@ -9,40 +7,43 @@ from devpi_common.viewhelp import iter_toxresults
 from devpi_server.log import threadlog as log
 from devpi_server.readonly import SeqViewReadonly
 from devpi_server.views import StatusView, url_for_entrypath
+from devpi_server.views import PyPIView
 from devpi_web.description import get_description
-from devpi_web.doczip import Docs, unpack_docs
+from devpi_web.doczip import Docs
+from devpi_web.doczip import docs_exist
+from devpi_web.doczip import docs_file_content
+from devpi_web.doczip import docs_file_exists
+from devpi_web.doczip import docs_file_path
 from devpi_web.indexing import is_project_cached
 from devpi_web.main import navigation_version
 from email.utils import parsedate
+from io import TextIOWrapper
+from html import escape
 from operator import attrgetter, itemgetter
 from py.xml import html
-try:
-    from pyramid.compat import decode_path_info
-except ImportError:
-    # pyramid is >= 2.0, which means we can assume Python 3
-    # see PEP 3333 for why we encode WSGI PATH_INFO to latin-1 before
-    # decoding it to utf-8
-    def decode_path_info(path):
-        return path.encode('latin-1').decode('utf-8')
 from pyramid.decorator import reify
 from pyramid.httpexceptions import HTTPBadGateway, HTTPError
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
+from pyramid.httpexceptions import HTTPNotModified
 from pyramid.httpexceptions import default_exceptionresponse_view
 from pyramid.interfaces import IRoutesMapper
 from pyramid.response import FileResponse, Response
 from pyramid.view import notfound_view_config, view_config
 from time import gmtime
-try:
-    from xmlrpc.client import Fault, Unmarshaller, dumps
-except ImportError:
-    from xmlrpclib import Fault, Unmarshaller, dumps
+from xmlrpc.client import Fault, Unmarshaller, dumps
 import functools
 import json
-import py
+import mimetypes
 
 seq_types = (list, tuple, SeqViewReadonly)
 
 
+# see PEP 3333 for why we encode WSGI PATH_INFO to latin-1 before
+# decoding it to utf-8
+def decode_path_info(path):
+    return path.encode('latin-1').decode('utf-8')
+
+
 class ContextWrapper(object):
     def __init__(self, context):
         self.context = context
@@ -94,6 +95,27 @@ class ContextWrapper(object):
                 "%s-%s is not registered" % (self.project, self.version))
 
 
+# wrapper to make future changes easier
+def ServerViews(context, request):
+    result = PyPIView(request)
+    result.context = context
+    return result
+
+
+def tween_trailing_slash_redirect(handler, registry):
+    def request_trailing_slash_redirect_handler(request):
+        if request.path == "/":
+            return handler(request)
+        if request.method != "GET":
+            return handler(request)
+        if '/+' in request.url:
+            return handler(request)
+        if not request.url.endswith('/'):
+            return handler(request)
+        return HTTPFound(location=request.url.rstrip('/'))
+    return request_trailing_slash_redirect_handler
+
+
 def get_doc_info(context, request, version=None, check_content=True):
     relpath = request.matchdict['relpath']
     if not relpath:
@@ -107,7 +129,7 @@ def get_doc_info(context, request, version=None, check_content=True):
         versions = context.stable_versions
     else:
         versions = [version]
-    doc_path = None
+    entry = None
     for doc_version in versions:
         try:
             linkstore = context.stage.get_linkstore_perstage(name, doc_version)
@@ -116,31 +138,60 @@ def get_doc_info(context, request, version=None, check_content=True):
         links = linkstore.get_links(rel='doczip')
         if not links:
             continue
-        doc_path = unpack_docs(context.stage, name, doc_version, links[0].entry)
-        if doc_path.isdir():
+        if not check_content or docs_exist(context.stage, name, doc_version, links[0].entry):
+            entry = links[0].entry
             break
-    if doc_path is None or not doc_path.isdir():
+    if entry is None:
         if version == 'stable':
             raise HTTPNotFound("No stable documentation available.")
         raise HTTPNotFound("No documentation available.")
-    doc_path = doc_path.join(relpath)
-    if check_content and not doc_path.check():
-        raise HTTPNotFound("File %s not found in documentation." % relpath)
-    return dict(
-        doc_path=doc_path,
+    if check_content:
+        relpath_exists = docs_file_exists(
+            context.stage, name, doc_version, entry, relpath)
+        if not relpath_exists:
+            raise HTTPNotFound("File %s not found in documentation." % relpath)
+    result = dict(
+        etag=entry.hash_value,
         relpath=relpath,
         doc_version=doc_version,
         version_mismatch=doc_version != navigation_version(context))
+    if check_content:
+        result['body'] = docs_file_content(
+            context.stage, name, doc_version, entry, relpath)
+        result['doc_path'] = docs_file_path(
+            context.stage, name, doc_version, entry, relpath)
+    return result
+
+
+def _guess_type(path):
+    (content_type, content_encoding) = mimetypes.guess_type(path, strict=False)
+    if content_type is None:
+        content_type = 'application/octet-stream'
+    return (content_type, content_encoding)
 
 
 @view_config(route_name="docroot", request_method="GET")
 def doc_serve(context, request):
     """ Serves the raw documentation files. """
+    if_none_match = request.if_none_match
+    if if_none_match:
+        doc_info = get_doc_info(context, request, check_content=False)
+        if doc_info['etag'] in if_none_match.etags:
+            return HTTPNotModified()
     context = ContextWrapper(context)
     doc_info = get_doc_info(context, request)
-    response = FileResponse(str(doc_info['doc_path']))
-    if context.version in ('latest', 'stable'):
-        response.cache_expires()
+    if doc_info['doc_path'] is None:
+        relpath = request.matchdict['relpath']
+        (content_type, content_encoding) = _guess_type(relpath)
+        response = Response(
+            body=doc_info['body'],
+            conditional_response=True,
+            content_type=content_type,
+            content_encoding=content_encoding)
+    else:
+        response = FileResponse(str(doc_info['doc_path']))
+    response.etag = doc_info['etag']
+    response.cache_expires(60)
     return response
 
 
@@ -191,7 +242,7 @@ def doc_show(context, request):
 
 @notfound_view_config(renderer="templates/notfound.pt")
 def notfound(request):
-    if request.method == 'POST':
+    if request.method not in ('GET', 'HEAD'):
         request.response.status = 404
         return dict(msg=request.exception)
     path = decode_path_info(request.environ['PATH_INFO'] or '/')
@@ -324,8 +375,8 @@ def get_files_info(request, linkstore, show_toxresults=False):
 
 
 def load_toxresult(link):
-    data = link.entry.file_get_content().decode("utf-8")
-    return json.loads(data)
+    with TextIOWrapper(link.entry.file_open_read(), encoding="utf-8") as f:
+        return json.load(f)
 
 
 def get_toxresults_info(linkstore, for_link, newest=True):
@@ -374,8 +425,7 @@ def get_docs_info(request, stage, linkstore):
     if not links:
         return
     name, ver = normalize_name(linkstore.project), linkstore.version
-    doc_path = unpack_docs(stage, name, ver, links[0].entry)
-    if doc_path.isdir():
+    if docs_exist(stage, name, ver, links[0].entry):
         return dict(
             title="%s-%s" % (name, ver),
             url=request.route_url(
@@ -410,6 +460,7 @@ def get_user_info(context, request, user):
 
 @view_config(
     route_name='root',
+    request_method="GET",
     renderer='templates/root.pt')
 def root(context, request):
     rawusers = sorted(
@@ -516,6 +567,46 @@ def index_get(context, request):
     return result
 
 
+def add_simple_page_navlink(request, context, nav_links):
+    nav_links.append(dict(
+        title="Simple index",
+        url=request.route_url(
+            "/{user}/{index}/+simple/{project}",
+            user=context.username, index=context.index, project=context.project)))
+
+
+def add_mirror_page_navlink(request, context, whitelist_info, nav_links):
+    if whitelist_info['has_mirror_base']:
+        for base in reversed(list(context.stage.sro())):
+            if base.ixconfig["type"] != "mirror":
+                continue
+            mirror_web_url_fmt = base.ixconfig.get("mirror_web_url_fmt")
+            if not mirror_web_url_fmt:
+                continue
+            nav_links.append(dict(
+                title="%s page" % base.ixconfig.get("title", "Mirror"),
+                url=mirror_web_url_fmt.format(name=context.verified_project)))
+
+
+def _index_refresh_form(request, stage, project):
+    url = request.route_url(
+        "project_refresh",
+        user=stage.username, index=stage.index, project=project)
+    title = "Refresh" if stage.ixconfig["type"] == "mirror" else "Refresh mirror links"
+    submit = '<input name="refresh" type="submit" value="%s"/>' % title
+    return '<form action="%s" method="post">%s</form>' % (url, submit)
+
+
+@view_config(route_name="project_refresh", request_method="POST")
+def project_refresh(context, request):
+    result = ServerViews(context, request).simple_refresh()
+    if isinstance(result, HTTPFound):
+        return HTTPFound(location=request.route_url(
+            "/{user}/{index}/{project}",
+            user=context.username, index=context.index, project=context.project))
+    return result
+
+
 @view_config(
     route_name="/{user}/{index}/{project}",
     accept="text/html", request_method="GET",
@@ -570,6 +661,8 @@ def project_get(context, request):
                 _release=None)
         else:
             version_info[version]['docs'] = docs
+    nav_links = []
+    add_simple_page_navlink(request, context, nav_links)
     versions = []
     for version in get_sorted_versions(version_info):
         versions.append(version_info[version])
@@ -579,21 +672,27 @@ def project_get(context, request):
         whitelist_info = dict(
             has_mirror_base=context.stage.has_mirror_base(context.project),
             blocked_by_mirror_whitelist=None)
+    add_mirror_page_navlink(request, context, whitelist_info, nav_links)
     stage = context.stage
     latest_verdata = {}
     latest_version = stage.get_latest_version_perstage(context.project)
     if latest_version is not None:
         latest_verdata = stage.get_versiondata_perstage(
             context.project, latest_version)
+    refresh_form = None
+    if whitelist_info['has_mirror_base']:
+        refresh_form = _index_refresh_form(request, stage, name)
     return dict(
         _context=context,
         title="%s/: %s versions" % (context.stage.name, context.project),
         blocked_by_mirror_whitelist=whitelist_info['blocked_by_mirror_whitelist'],
+        nav_links=nav_links,
         latest_version=latest_version,
         latest_url=request.route_url(
             "/{user}/{index}/{project}/{version}",
             user=user, index=index, project=name, version='latest'),
         latest_version_data=latest_verdata,
+        refresh_form=refresh_form,
         versions=versions)
 
 
@@ -625,8 +724,8 @@ def version_get(context, request):
         else:
             if not value:
                 continue
-            value = py.xml.escape(value)
-        infos.append((py.xml.escape(key), value))
+            value = escape(value)
+        infos.append((escape(key), value))
     show_toxresults = (stage.ixconfig['type'] != 'mirror')
     linkstore = stage.get_linkstore_perstage(name, version)
     files = get_files_info(request, linkstore, show_toxresults)
@@ -641,27 +740,14 @@ def version_get(context, request):
         nav_links.append(dict(
             title="Homepage",
             url=home_page))
-    nav_links.append(dict(
-        title="Simple index",
-        url=request.route_url(
-            "/{user}/{index}/+simple/{project}",
-            user=context.username, index=context.index, project=context.project)))
+    add_simple_page_navlink(request, context, nav_links)
     if hasattr(stage, 'get_mirror_whitelist_info'):
         whitelist_info = stage.get_mirror_whitelist_info(name)
     else:
         whitelist_info = dict(
             has_mirror_base=stage.has_mirror_base(name),
             blocked_by_mirror_whitelist=False)
-    if whitelist_info['has_mirror_base']:
-        for base in reversed(list(stage.sro())):
-            if base.ixconfig["type"] != "mirror":
-                continue
-            mirror_web_url_fmt = base.ixconfig.get("mirror_web_url_fmt")
-            if not mirror_web_url_fmt:
-                continue
-            nav_links.append(dict(
-                title="%s page" % base.ixconfig.get("title", "Mirror"),
-                url=mirror_web_url_fmt.format(name=name)))
+    add_mirror_page_navlink(request, context, whitelist_info, nav_links)
     cmp_version = Version(version)
     if context._stable_versions:
         stable_version = Version(context._stable_versions[0])
@@ -688,7 +774,7 @@ def version_get(context, request):
         nav_links=nav_links,
         infos=infos,
         metadata_list_fields=frozenset(
-            py.xml.escape(x)
+            escape(x)
             for x in getattr(stage, 'metadata_list_fields', ())),
         files=files,
         blocked_by_mirror_whitelist=whitelist_info['blocked_by_mirror_whitelist'],
@@ -748,6 +834,7 @@ def toxresult(context, request):
 @view_config(
     route_name="/+status",
     accept="text/html",
+    request_method="GET",
     renderer="templates/status.pt")
 def statusview(request):
     status = StatusView(request)._status()
@@ -1016,6 +1103,7 @@ class SearchView:
 
     @view_config(
         route_name='search_help',
+        request_method="GET",
         renderer='templates/search_help.pt')
     def search_help(self):
         return dict()
@@ -1036,6 +1124,9 @@ class SearchView:
         log.debug("xmlrpc_search {0}".format((fields, operator)))
         return dict(fields=fields, operator=operator)
 
+    @view_config(
+        route_name="/{user}/{index}", request_method="POST",
+        content_type="text/xml", is_mutating=False)
     @view_config(
         route_name="/{user}/{index}/", request_method="POST",
         content_type="text/xml", is_mutating=False)
