@@ -1,4 +1,3 @@
-from __future__ import unicode_literals
 from collections import defaultdict
 from devpi_common.validation import normalize_name
 from devpi_server.log import threadlog as log
@@ -226,10 +225,7 @@ class IndexingSharedData(object):
     ERROR_QUEUE_MAX_DELAY = 60 * 60
 
     def __init__(self):
-        try:
-            from queue import Empty, PriorityQueue
-        except ImportError:
-            from Queue import Empty, PriorityQueue
+        from queue import Empty, PriorityQueue
         self.Empty = Empty
         self.queue = PriorityQueue()
         self.error_queue = PriorityQueue()
@@ -295,10 +291,11 @@ class IndexingSharedData(object):
                     "Currently in %s" % (processed, queued, project.indexname))
             if project.is_from_mirror:
                 # we find the last serial the project was changed to avoid re-indexing
+                name = normalize_name(project.name)
                 project_serial = project.stage.get_last_project_change_serial_perstage(
-                    project.name, at_serial=at_serial)
+                    name, at_serial=at_serial)
                 # mirrors have no docs, so we can shortcut
-                path = '/%s/%s' % (project.indexname, project.name)
+                path = '/%s/%s' % (project.indexname, name)
                 existing = None
                 doc_num = searcher.document_number(path=path)
                 if doc_num is not None:
@@ -809,6 +806,7 @@ class Index(object):
         for item in result['items']:
             data = item['data']
             (user, index, name) = data['path'][1:].split('/')
+            name = data.get('name', name)
             stage = stagename2stage.get('%s/%s' % (user, index))
             if stage is None:
                 continue
