@@ -1,10 +1,10 @@
-from devpi_server import __version__ as devpi_server_version
-from pkg_resources import parse_version
+from devpi_common.metadata import parse_version
+from devpi_server import __version__ as _devpi_server_version
 import pytest
 import textwrap
 
 
-devpi_server_version = parse_version(devpi_server_version)
+devpi_server_version = parse_version(_devpi_server_version)
 
 
 def test_importable():
@@ -14,12 +14,14 @@ def test_importable():
 
 def test_pkgresources_version_matches_init():
     import devpi_web
-    import pkg_resources
+    try:
+        import importlib.metadata as importlib_metadata
+    except ImportError:
+        import importlib_metadata as importlib_metadata
     ver = devpi_web.__version__
-    assert pkg_resources.get_distribution("devpi_web").version == ver
+    assert importlib_metadata.version("devpi-web") == ver
 
 
-@pytest.mark.nomockprojectsremote
 def test_devpi_mirror_initialnames(caplog, pypistage):
     import logging
     caplog.set_level(logging.NOTSET)
@@ -36,11 +38,40 @@ def test_devpi_mirror_initialnames(caplog, pypistage):
     assert len(logs) == 1
 
 
+@pytest.mark.skipif(
+    devpi_server_version < parse_version("6.6.0dev"),
+    reason="Needs un-normalized project names from list_projects_perstage on mirrors")
+def test_devpi_mirror_initialnames_original_name(caplog, pypistage):
+    import logging
+    caplog.set_level(logging.NOTSET)
+    from devpi_common.validation import normalize_name
+    from devpi_web.main import devpiserver_mirror_initialnames
+    from devpi_web.main import get_indexer
+    projects = set(["Django", "pytest", "ploy_ansible"])
+    pypistage.mock_simple_projects(projects)
+    pypistage.mock_simple(
+        "Django", pypiserial=10,
+        pkgver="django-1.0.zip")
+    indexer = get_indexer(pypistage.xom)
+    pypistage.xom.thread_pool.start_one(indexer.indexer_thread)
+    with pypistage.keyfs.transaction():
+        devpiserver_mirror_initialnames(pypistage, pypistage.list_projects_perstage())
+    indexer.indexer_thread.wait()
+    for project in projects:
+        (item1,) = indexer.query_projects(project)['items']
+        (item2,) = indexer.query_projects(normalize_name(project))['items']
+        data1 = item1['data']
+        data2 = item2['data']
+        assert data1 == data2
+        assert data1['name'] == project
+
+
 def test_devpi_stage_created(monkeypatch, pypistage, mock):
     from devpi_web.main import devpiserver_stage_created
     list_projects_perstage = mock.MagicMock()
     list_projects_perstage.return_value = []
-    monkeypatch.setattr(pypistage, "list_projects_perstage", list_projects_perstage)
+    monkeypatch.setattr(
+        pypistage.__class__, "list_projects_perstage", list_projects_perstage)
     with pypistage.keyfs.transaction():
         devpiserver_stage_created(pypistage)
     assert list_projects_perstage.called
