@@ -1,14 +1,12 @@
-# -*- coding: utf-8 -*-
 from devpi_common.archive import zip_dict
-from devpi_server import __version__ as devpi_server_version
-from pkg_resources import parse_version
+from devpi_common.metadata import parse_version
+from devpi_server import __version__ as _devpi_server_version
 from time import struct_time
-import py
 import pytest
 import re
 
 
-devpi_server_version = parse_version(devpi_server_version)
+devpi_server_version = parse_version(_devpi_server_version)
 
 
 def compareable_text(text):
@@ -36,6 +34,10 @@ def test_root_view_with_index(mapp, testapp):
         (api.stagename, "http://localhost/%s" % api.stagename)]
 
 
+def test_root_view_post(testapp):
+    testapp.post('/', headers=dict(accept="text/html"), code=404)
+
+
 def test_user_view_root_pypi(testapp):
     r = testapp.get('/root', headers=dict(accept="text/html"))
     assert r.status_code == 200
@@ -129,7 +131,7 @@ def test_index_view_project_files(mapp, testapp):
 
 
 @pytest.mark.with_notifier
-def test_index_view_project_docs(mapp, testapp):
+def test_index_view_project_docs(keep_docs_packed, mapp, testapp):
     api = mapp.create_and_use(indexconfig=dict(bases=["root/pypi"]))
     mapp.set_versiondata({"name": "pkg1", "version": "2.6"})
     content = zip_dict({"index.html": "<html/>"})
@@ -198,6 +200,7 @@ def test_project_view(mapp, testapp):
     assert r.status_code == 200
     links = r.html.select('#content a')
     assert [(l.text, l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/pkg-name" % api.stagename),
         ("2.7", "http://localhost/%s/pkg-name/latest" % api.stagename),
         (api.stagename, "http://localhost/%s" % api.stagename),
         ("2.7", "http://localhost/%s/pkg-name/2.7" % api.stagename),
@@ -214,6 +217,7 @@ def test_project_projectname_redirect(mapp, testapp):
     r = testapp.xget(200, api.index + '/pkg_hello', headers=dict(accept="text/html"))
     links = r.html.select('#content a')
     assert [(l.text, l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/pkg-hello" % api.stagename),
         ("1.0", "http://localhost/%s/pkg-hello/latest" % api.stagename),
         (api.stagename, "http://localhost/%s" % api.stagename),
         ("1.0", "http://localhost/%s/pkg-hello/1.0" % api.stagename)]
@@ -232,7 +236,7 @@ def test_project_not_found(mapp, testapp):
 
 
 @pytest.mark.with_notifier
-def test_project_view_docs_only(mapp, testapp):
+def test_project_view_docs_only(keep_docs_packed, mapp, testapp):
     api = mapp.create_and_use()
     content = zip_dict({"index.html": "<html/>"})
     mapp.set_versiondata({"name": "pkg1", "version": "2.6"})
@@ -253,6 +257,8 @@ def test_project_view_root_pypi(mapp, testapp, pypistage):
     assert r.status_code == 200
     links = r.html.select('#content a')
     assert [(l.text, l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/pkg1" % pypistage.name),
+        ("PyPI page", "https://pypi.org/project/pkg1/"),
         ("2.7", "http://localhost/root/pypi/pkg1/latest"),
         ("root/pypi", "http://localhost/root/pypi"),
         ("2.7", "http://localhost/root/pypi/pkg1/2.7"),
@@ -265,7 +271,9 @@ def test_project_view_root_pypi_no_releases(mapp, testapp, pypistage):
     r = testapp.get('/root/pypi/pkg1', headers=dict(accept="text/html"))
     assert r.status_code == 200
     links = r.html.select('#content a')
-    assert [(l.text, l.attrs['href']) for l in links] == []
+    assert [(l.text, l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/pkg1" % pypistage.name),
+        ("PyPI page", "https://pypi.org/project/pkg1/")]
 
 
 def test_project_view_root_pypi_external_link_bad_name(mapp, testapp, pypistage):
@@ -280,6 +288,8 @@ def test_project_view_root_pypi_external_link_bad_name(mapp, testapp, pypistage)
     assert r.status_code == 200
     links = r.html.select('#content a')
     assert [(l.text, l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/pkg1" % pypistage.name),
+        ("PyPI page", "https://pypi.org/project/pkg1/"),
         ("2.7", "http://localhost/root/pypi/pkg1/latest"),
         ("root/pypi", "http://localhost/root/pypi"),
         ("2.7", "http://localhost/root/pypi/pkg1/2.7"),
@@ -288,7 +298,7 @@ def test_project_view_root_pypi_external_link_bad_name(mapp, testapp, pypistage)
 
 
 @pytest.mark.with_notifier
-def test_project_view_root_and_docs(mapp, testapp, pypistage):
+def test_project_view_root_and_docs(keep_docs_packed, mapp, testapp, pypistage):
     pypistage.mock_simple("pkg1", text='''
             <a href="../../pkg/pkg1-2.7.zip" />
             <a href="../../pkg/pkg1-2.6.zip" />
@@ -303,6 +313,8 @@ def test_project_view_root_and_docs(mapp, testapp, pypistage):
     r = testapp.xget(200, api.index + '/pkg1', headers=dict(accept="text/html"))
     links = r.html.select('#content a')
     assert [(l.text, l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/pkg1" % api.stagename),
+        ("PyPI page", "https://pypi.org/project/pkg1/"),
         ("2.6", "http://localhost/user1/dev/pkg1/latest"),
         ("root/pypi", "http://localhost/root/pypi"),
         ("2.7", "http://localhost/root/pypi/pkg1/2.7"),
@@ -362,9 +374,7 @@ def test_version_view(mapp, testapp, monkeypatch):
     description = r.html.select('#description')
     assert len(description) == 1
     description = description[0]
-    assert py.builtin._totext(
-        description.renderContents().strip(),
-        'utf-8') == u'<p>föö</p>'
+    assert description.renderContents().strip().decode('utf-8') == u'<p>föö</p>'
     filesinfo = [
         tuple(
             compareable_text(t.text).split()
@@ -405,9 +415,7 @@ def test_markdown_description_without_content_type(mapp, testapp, monkeypatch):
 
     description = r.html.select('#description')
     assert len(description) == 1
-    assert '#' in py.builtin._totext(
-        description[0].renderContents().strip(),
-        'utf-8')
+    assert '#' in description[0].renderContents().strip().decode('utf-8')
 
 
 @pytest.mark.with_notifier
@@ -427,9 +435,7 @@ def test_markdown_description_with_content_type(mapp, testapp, monkeypatch):
 
     description = r.html.select('#description')
     assert len(description) == 1
-    assert py.builtin._totext(
-        description[0].renderContents().strip(),
-        'utf-8') == u'<h1>Description</h1>'
+    assert description[0].renderContents().strip().decode('utf-8') == u'<h1>Description</h1>'
 
 
 @pytest.mark.with_notifier
@@ -451,12 +457,12 @@ def test_description_updated(mapp, testapp):
         "name": "pkg-hello", "version": "1.0", "description": "foo"})
     r = testapp.xget(200, api.index + "/pkg-hello/1.0", headers=dict(accept="text/html"))
     description, = r.html.select('#description')
-    assert '<p>foo</p>' == py.builtin._totext(description.renderContents().strip(), 'utf-8')
+    assert '<p>foo</p>' == description.renderContents().strip().decode('utf-8')
     mapp.set_versiondata({
         "name": "pkg-hello", "version": "1.0", "description": "bar"})
     r = testapp.xget(200, api.index + "/pkg-hello/1.0", headers=dict(accept="text/html"))
     description, = r.html.select('#description')
-    assert '<p>bar</p>' == py.builtin._totext(description.renderContents().strip(), 'utf-8')
+    assert '<p>bar</p>' == description.renderContents().strip().decode('utf-8')
 
 
 @pytest.mark.with_notifier
@@ -466,7 +472,7 @@ def test_description_empty(mapp, testapp):
         "name": "pkg-hello", "version": "1.0"})
     r = testapp.xget(200, api.index + "/pkg-hello/1.0", headers=dict(accept="text/html"))
     description, = r.html.select('#description')
-    assert '<p>No description in metadata</p>' == py.builtin._totext(description.renderContents().strip(), 'utf-8')
+    assert '<p>No description in metadata</p>' == description.renderContents().strip().decode('utf-8')
 
 
 def test_version_not_found(mapp, testapp):
@@ -601,8 +607,8 @@ def test_version_view_latest_stable(mapp, testapp):
 
 def test_complex_name(mapp, testapp):
     from devpi_common import __version__
-    import pkg_resources
-    if pkg_resources.parse_version(__version__) < pkg_resources.parse_version('3.2.0dev'):
+    from devpi_common.metadata import parse_version
+    if parse_version(__version__) < parse_version('3.2.0dev'):
         pytest.skip("Only works with devpi-common >= 3.2.0")
     api = mapp.create_and_use()
     pkgname = "my-binary-package-name-1-4-3-yip"
@@ -620,6 +626,7 @@ def test_complex_name(mapp, testapp):
         200, api.index + '/%s' % pkgname, headers=dict(accept="text/html"))
     links = r.html.select('#content a')
     assert [(compareable_text(l.text), l.attrs['href']) for l in links] == [
+        ("Simple index", "http://localhost/%s/+simple/%s" % (api.stagename, pkgname)),
         ('0.9', 'http://localhost/user1/dev/%s/latest' % pkgname),
         ('user1/dev', 'http://localhost/user1/dev'),
         ('0.9', 'http://localhost/user1/dev/%s/0.9' % pkgname)]
